#ifndef _HYBRIDARRAY_H_
#define _HYBRIDARRAY_H_

namespace Tools
{

	#define MacroSafeDelete(macroPointer) { if(macroPointer != 0) {delete (macroPointer); (macroPointer)=0;} }
	#define MacroSafeDeleteArray(macroPointer) { if(macroPointer != 0) {delete[] (macroPointer); (macroPointer)=0;} }

	template <class Type,class CountType =unsigned short,int MaxSize=20, int ItemIncreaseSize=10,bool UseSafeModeFlag=false>
	class HybridArray
	{
	public:
		HybridArray();
		HybridArray(const HybridArray& aHybridArray);
		~HybridArray();

		HybridArray& operator=(const HybridArray& aHybridArray);
	
		inline Type& operator[](const CountType& aIndex) const;

		inline void Add(const Type& aObject);
		inline void Insert(CountType aIndex,Type& aObject);
		inline void DeleteCyclic(Type& aObject);
		inline void DeleteCyclicAtIndex(CountType aItemNumber);
		inline void RemoveCyclic(const Type& aObject);
		inline void RemoveCyclicAtIndex(CountType aItemNumber);


		inline void RemoveAll();
		inline void DeleteAll();

		void Optimize();
		__forceinline CountType Count() const;

	protected:
		inline void ReSize(int aNewSize);

		CountType myCurrentSize;
		CountType myMaxSize;
		CountType myIncreaseSize;
		Type myStaticArray[MaxSize];
		Type *myArrayPointer;
		bool mySafeMode;
		bool myGrowing;
	};
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::HybridArray()
{
	myCurrentSize = 0;
	myMaxSize = MaxSize;
	myIncreaseSize = ItemIncreaseSize;
	assert ( (myIncreaseSize != 0) && "HybridArray: Constructor() - instanced object with IncreaseSize 0");
	myArrayPointer = myStaticArray;
	mySafeMode = UseSafeModeFlag;
	myGrowing = false;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::HybridArray
	(const Tools::HybridArray<Type,CountType,MaxSize,ItemIncreaseSize,UseSafeModeFlag> &aHybridArray)
{
	myCurrentSize = aHybridArray.myCurrentSize;
	myMaxSize = aHybridArray.myMaxSize;
	myIncreaseSize = aHybridArray.myIncreaseSize;
	mySafeMode = aHybridArray.mySafeMode;
	myGrowing = aHybridArray.myGrowing;

	if ( myGrowing == true )
	{
		myGrowingArray = new Type[myMaxSize];
		if ( mySafeMode == true )
		{
			int bytesToCopy = sizeof(Type) * myCurrentSize;
			memcpy(myGrowingArray, aGrowingArray.myGrowingArray, bytesToCopy);
		}
		else
		{
			for ( unsigned int positionCopied = 0; positionCopied < myCurrentSize; positionCopied++)
			{
				myGrowingArray[positionCopied] = aHybridArray.myGrowingArray[positionCopied];
			}
		}
		myArrayPointer = &myGrowingArray;
	}
	else
	{
		myStaticArray = aHybridArray.myStaticArray;
		myArrayPointer = &myStaticArray;
	}
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::~HybridArray()
{
	for ( unsigned int positionCleared = 0; positionCleared < MaxSize; positionCleared++)
	{
		myStaticArray[positionCleared] = 0;	
	};
	MacroSafeDeleteArray(myArrayPointer);
	
	myCurrentSize = 0;
	myMaxSize = 0;
	myIncreaseSize = 0;
	mySafeMode = 0;
	myGrowing = 0;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>& 
Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::operator =
(const Tools::HybridArray<Type,CountType,MaxSize,ItemIncreaseSize,UseSafeModeFlag> &aHybridArray)
{
	if ( myGrowing == true )
	{
		MacroSafeDeleteArray(myArrayPointer);
	}

	myCurrentSize = aHybridArray.myCurrentSize;
	myMaxSize = aHybridArray.myMaxSize;
	myIncreaseSize = aHybridArray.myIncreaseSize;
	mySafeMode = aHybridArray.mySafeMode;
	myGrowing = aHybridArray.myGrowing;

	if( aHybridArray.myGrowing == true )
	{
		myArrayPointer = new Type[myMaxSize];
		if ( mySafeMode == true )
		{
			int bytesToCopy = sizeof(Type) * myCurrentSize;
			memcpy(myArrayPointer, aHybridArray.myArrayPointer, bytesToCopy);
		}
		else
		{
			for ( unsigned int positionCopied = 0; positionCopied < myCurrentSize; positionCopied++)
			{
				myArrayPointer[positionCopied] = aHybridArray.myArrayPointer[positionCopied];
			}
		}
	}
	else
	{
		for ( unsigned int positionCopied = 0; positionCopied < myCurrentSize; positionCopied++)
		{
			myStaticArray[positionCopied] = aHybridArray.myStaticArray[positionCopied];
		}
		myArrayPointer = myStaticArray;
	}

	return *this;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline Type& Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::operator []
(const CountType &aIndex) const
{
	assert( (aIndex >= 0 && aIndex <= (myCurrentSize-1)) && "HybridArray: operator [] - Out of bounds.");
	assert ( (myCurrentSize < myMaxSize) && "HybridArray: operator [] - Size of array bigger than max size!");

	return myArrayPointer[aIndex];
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::ReSize
(int aNewSize)
{
	if ( myGrowing == false )
	{
		myArrayPointer = new Type[aNewSize];
		if ( mySafeMode == true )
		{
			int bytesToCopy = sizeof(Type) * myCurrentSize;
			memcpy(myArrayPointer, myStaticArray, bytesToCopy);
		}
		else
		{
			for (unsigned int copiedPosition = 0; copiedPosition < myCurrentSize; copiedPosition++)
			{
				myArrayPointer[copiedPosition] = myStaticArray[copiedPosition];
			}
		}
		myMaxSize += myIncreaseSize;
		myGrowing = true;
	}
	else
	{
		Type * const temp = new Type[aNewSize];
		if ( mySafeMode == true )
		{
			int bytesToCopy = sizeof(Type) * myCurrentSize;
			memcpy(temp, myArrayPointer, bytesToCopy);
		}
		else
		{
			for (unsigned int copiedPosition = 0; copiedPosition < myCurrentSize; copiedPosition++)
			{
				temp[copiedPosition] = myArrayPointer[copiedPosition];
			}
		}
		MacroSafeDeleteArray(myArrayPointer);
		myArrayPointer = temp;
		myMaxSize += myIncreaseSize;
	}
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::Add
(const Type &aObject)
{
	if (myCurrentSize == myMaxSize)
	{
		ReSize(myMaxSize + myIncreaseSize);
	}

	myArrayPointer[myCurrentSize] = aObject;
	myCurrentSize++;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::Insert
(CountType aIndex, Type &aObject)
{
	assert ( (aIndex >= 0 && aIndex <= (myCurrentSize-1)) && "HybridArray: Insert - Out of Bounds.");
	assert ( (myCurrentSize < myMaxSize) && "HybridArray: Insert - Size of array bigger than max size!");

	if ( myCurrentSize == myMaxSize )
	{
		ReSize(myMaxSize + myIncreaseSize);
	}

	for ( int objectMoved = (myCurrentSize-1); objectMoved >= aIndex; objectMoved--)
	{
		myArrayPointer[objectMoved+1] = myArrayPointer[objectMoved];
	}

	myArrayPointer[aIndex] = aObject;
	myCurrentSize++;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::DeleteCyclic
(Type &aObject)
{
	bool objectFound = false;
	for ( int objectChecked = 0; objectChecked < myCurrentSize; objectChecked++)
	{
		if ( aObject == myArrayPointer[objectChecked] )
		{
			objectFound = true;
			delete(myArrayPointer[objectChecked]);
			myArrayPointer[objectChecked] = myArrayPointer[myCurrentSize-1];
			myCurrentSize--;
			break;
		}
	}

	assert( (objectFound==true) &&"GrowingArray: DeleteCyclic - object not found in GrowingArray.");
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::DeleteCyclicAtIndex
(CountType aItemNumber)
{
	assert ( (aItemNumber >= 0 && aItemNumber <= (myCurrentSize-1)) && "HybridArray: DeleteCyclicAtIndex - Out of Bounds.");
	assert ( (myCurrentSize < myMaxSize) && "HybridArray: DeleteCyclicAtIndex - Size of array bigger than max size!");

	delete(myArrayPointer[aItemNumber]);
	myArrayPointer[aItemNumber] = myArrayPointer[myCurrentSize-1];
	myCurrentSize--;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::RemoveCyclic
(const Type &aObject)
{
	bool objectFound = false;
	for ( int objectChecked = 0; objectChecked < myCurrentSize; objectChecked++)
	{
		if ( aObject == myArrayPointer[objectChecked] )
		{
			objectFound = true;
			myArrayPointer[objectChecked] = myArrayPointer[myCurrentSize-1];
			myArrayPointer[myCurrentSize-1] = 0;
			myCurrentSize--;
			break;
		}
	}

	assert( (objectFound==true) &&"GrowingArray: RemoveCyclic - object not found in GrowingArray.");
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::RemoveCyclicAtIndex
(CountType aItemNumber)
{
	assert ( (aItemNumber >= 0 && aItemNumber <= (myCurrentSize-1)) && "HybridArray: RemoveCyclicAtIndex - Out of Bounds.");
	assert ( (myCurrentSize < myMaxSize) && "HybridArray: RemoveCyclicAtIndex - Size of array bigger than max size!");

	myArrayPointer[aItemNumber] = myArrayPointer[myCurrentSize-1];
	myArrayPointer[myCurrentSize-1] = 0;
	myCurrentSize--;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::RemoveAll()
{
	assert ( (myCurrentSize < myMaxSize) && "HybridArray: RemoveAll - Size of array bigger than max size!");
	myCurrentSize = 0;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
inline void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::DeleteAll()
{
	assert ( (myCurrentSize < myMaxSize) && "HybridArray: DeleteAll - Size of array bigger than max size!");
	if ( myGrowing == true )
	{
		for ( unsigned int objectDeleted = 0; objectDeleted < myCurrentSize; objectDeleted++)
		{
			delete myArrayPointer[objectDeleted];
		}
	}

	myCurrentSize = 0;
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
void Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::Optimize()
{
	if ( myGrowing == false )
	{
		ReSize(myCurrentSize);
		myIncreaseSize = 1;
	}
}

template <class Type,class CountType,int MaxSize, int ItemIncreaseSize,bool UseSafeModeFlag>
__forceinline CountType Tools::HybridArray<Type, CountType , MaxSize, ItemIncreaseSize, UseSafeModeFlag>::Count() const
{
	return myCurrentSize;
}

#endif